@@ -32,7 +32,6 @@
 #include <KLocalizedString>
 #include <KPluginFactory>
 #include <KWallet>
-#include <KWindowSystem>
 
 SecretAgent::SecretAgent(QObject *parent)
     : NetworkManager::SecretAgent(QStringLiteral("org.kde.plasma.networkmanagement"), NetworkManager::SecretAgent::Capability::VpnHints, parent)
@@ -189,7 +188,8 @@ void SecretAgent::dialogAccepted()
                 } else if (completeConnectionSettings->connectionType() == NetworkManager::ConnectionSettings::Wireless) {
                     NetworkManager::WirelessSecuritySetting::Ptr wirelessSecuritySetting =
                         completeConnectionSettings->setting(NetworkManager::Setting::WirelessSecurity).staticCast<NetworkManager::WirelessSecuritySetting>();
-                    if (wirelessSecuritySetting && wirelessSecuritySetting->keyMgmt() == NetworkManager::WirelessSecuritySetting::WpaEap) {
+                    if (wirelessSecuritySetting && wirelessSecuritySetting->keyMgmt() == NetworkManager::WirelessSecuritySetting::WpaEap
+                        || wirelessSecuritySetting->keyMgmt() == NetworkManager::WirelessSecuritySetting::WpaEapSuiteB192) {
                         NetworkManager::Security8021xSetting::Ptr security8021xSetting =
                             completeConnectionSettings->setting(NetworkManager::Setting::Security8021x).staticCast<NetworkManager::Security8021xSetting>();
                         if (security8021xSetting) {
@@ -435,9 +435,8 @@ bool SecretAgent::processGetSecrets(SecretsRequest &request) const
         } else {
             request.dialog = m_dialog;
             request.saveSecretsWithoutReply = !connectionSettings->permissions().isEmpty();
+            m_dialog->setWindowFlag(Qt::WindowStaysOnTopHint);
             m_dialog->show();
-            KWindowSystem::setState(m_dialog->winId(), NET::KeepAbove);
-            KWindowSystem::forceActiveWindow(m_dialog->winId());
             return false;
         }
     } else if (isVpn && userRequested) { // just return what we have
