@@ -18,6 +18,7 @@
 
 #include <KLocalizedString>
 #include <KMessageBox>
+#include <kwidgetsaddons_version.h>
 
 #define NM_SETTING_BOND_OPTION_MII_MONITOR "mii"
 #define NM_SETTING_BOND_OPTION_ARP_MONITOR "arp"
@@ -238,14 +239,23 @@ void BondWidget::deleteBond()
 
     if (connection) {
         // qCDebug(PLASMA_NM_EDITOR_LOG) << "About to delete bonded connection" << currentItem->text() << uuid;
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        if (KMessageBox::questionTwoActions(this,
+#else
         if (KMessageBox::questionYesNo(this,
-                                       i18n("Do you want to remove the connection '%1'?", connection->name()),
-                                       i18n("Remove Connection"),
-                                       KStandardGuiItem::remove(),
-                                       KStandardGuiItem::no(),
-                                       QString(),
-                                       KMessageBox::Dangerous)
+
+#endif
+                                            i18n("Do you want to remove the connection '%1'?", connection->name()),
+                                            i18n("Remove Connection"),
+                                            KStandardGuiItem::remove(),
+                                            KStandardGuiItem::cancel(),
+                                            QString(),
+                                            KMessageBox::Dangerous)
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+            == KMessageBox::ButtonCode::PrimaryAction) {
+#else
             == KMessageBox::Yes) {
+#endif
             connection->remove();
             delete currentItem;
             slotWidgetChanged();
