@@ -23,25 +23,25 @@
 
 #include <libnm/nm-vpn-plugin-info.h>
 
-#if WITH_MODEMMANAGER_SUPPORT
 #include <ModemManagerQt/Manager>
 #include <ModemManagerQt/ModemDevice>
-#endif
 
 #include <QDBusError>
 #include <QDBusMetaType>
 #include <QDBusPendingReply>
 #include <QFile>
 #include <QIcon>
-#include <QPointer>
 
+#include <KIO/OpenUrlJob>
 #include <KLocalizedString>
 #include <KNotification>
+#include <KOSRelease>
 #include <KPluginMetaData>
 #include <KProcess>
 #include <KUser>
 #include <KWallet>
-#include <KWindowSystem>
+
+#include <nm-client.h>
 
 #define AGENT_SERVICE "org.kde.kded5"
 #define AGENT_PATH "/modules/networkmanagement"
@@ -98,40 +98,56 @@ void Handler::activateConnection(const QString &connection, const QString &devic
                 return md.value(QStringLiteral("X-NetworkManager-Services")) == vpnSetting->serviceType();
             };
 
-            const QVector<KPluginMetaData> plugins = KPluginMetaData::findPlugins(QStringLiteral("plasma/network/vpn"), filter);
+            const QVector<KPluginMetaData> plasmaNmPlugins = KPluginMetaData::findPlugins(QStringLiteral("plasma/network/vpn"), filter);
 
             const QString pluginBaseName = vpnSetting->serviceType().remove(QLatin1String("org.freedesktop.NetworkManager."));
-            bool pluginMissing = plugins.isEmpty();
-            QString errorMessage;
 
-            if (plugins.empty()) {
-                errorMessage = i18n("Plasma is missing support for '%1' VPN connections. Please report this to your distribution.", pluginBaseName);
-            }
-
-            // Check missing NetworkManager VPN plugin
-            if (!pluginMissing) {
-                GSList *plugins = nullptr;
-                plugins = nm_vpn_plugin_info_list_load();
-
-                NMVpnPluginInfo *plugin_info = nm_vpn_plugin_info_list_find_by_service(plugins, vpnSetting->serviceType().toStdString().c_str());
-                pluginMissing = !plugin_info;
-                errorMessage = i18n("NetworkManager is missing support for '%1' VPN connections. Please use the package manager to install it.", pluginBaseName);
-            }
-
-            if (pluginMissing) {
+            if (plasmaNmPlugins.empty()) {
                 qCWarning(PLASMA_NM_LIBS_LOG) << "VPN" << vpnSetting->serviceType() << "not found, skipping";
                 auto notification = new KNotification(QStringLiteral("MissingVpnPlugin"), KNotification::Persistent, this);
                 notification->setComponentName(QStringLiteral("networkmanagement"));
                 notification->setTitle(con->name());
-                notification->setText(errorMessage);
+                notification->setText(i18n("Plasma is missing support for '%1' VPN connections.", pluginBaseName));
                 notification->setIconName(QStringLiteral("dialog-error"));
+                notification->setActions({i18n("Report Bug")});
+                connect(notification, &KNotification::action1Activated, this, [notification] {
+                    auto *job = new KIO::OpenUrlJob(QUrl(KOSRelease().bugReportUrl()));
+                    job->setStartupId(notification->xdgActivationToken().toUtf8());
+                    job->start();
+                });
+
+                notification->sendEvent();
+
+                return;
+            }
+
+            // Check missing NetworkManager VPN plugin
+            GSList *networkManagerPlugins = nullptr;
+            networkManagerPlugins = nm_vpn_plugin_info_list_load();
+
+            NMVpnPluginInfo *plugin_info = nm_vpn_plugin_info_list_find_by_service(networkManagerPlugins, vpnSetting->serviceType().toStdString().c_str());
+
+            if (!plugin_info) {
+                qCWarning(PLASMA_NM_LIBS_LOG) << "VPN" << vpnSetting->serviceType() << "not found, skipping";
+                auto notification = new KNotification(QStringLiteral("MissingVpnPlugin"), KNotification::Persistent, this);
+                notification->setComponentName(QStringLiteral("networkmanagement"));
+                notification->setTitle(con->name());
+                notification->setText(i18n("NetworkManager is missing support for '%1' VPN connections.", pluginBaseName));
+                notification->setIconName(QStringLiteral("dialog-error"));
+                notification->setActions({i18n("Install")});
+
+                connect(notification, &KNotification::action1Activated, this, [notification, pluginBaseName] {
+                    auto *job = new KIO::OpenUrlJob(QUrl(QStringLiteral("appstream:network-manager-") + pluginBaseName));
+                    job->setStartupId(notification->xdgActivationToken().toUtf8());
+                    job->start();
+                });
+
                 notification->sendEvent();
                 return;
             }
         }
     }
 
-#if WITH_MODEMMANAGER_SUPPORT
     if (con->settings()->connectionType() == NetworkManager::ConnectionSettings::Gsm) {
         NetworkManager::ModemDevice::Ptr nmModemDevice = NetworkManager::findNetworkInterface(device).objectCast<NetworkManager::ModemDevice>();
         if (nmModemDevice) {
@@ -155,7 +171,6 @@ void Handler::activateConnection(const QString &connection, const QString &devic
             }
         }
     }
-#endif
 
     QDBusPendingReply<QDBusObjectPath> reply = NetworkManager::activateConnection(connection, device, specificObject);
     auto watcher = new QDBusPendingCallWatcher(reply, this);
@@ -164,8 +179,12 @@ void Handler::activateConnection(const QString &connection, const QString &devic
     connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
 }
 
-QString Handler::wifiCode(const QString &connectionPath, const QString &ssid, int _securityType) const
+void Handler::requestWifiCode(const QString &connectionPath, const QString &ssid, int _securityType, const QString &connectionName)
 {
+    if (!m_requestWifiCodeWatcher.isNull()) {
+        delete m_requestWifiCodeWatcher;
+    }
+
     auto securityType = static_cast<NetworkManager::WirelessSecurityType>(_securityType);
 
     QString ret = QStringLiteral("WIFI:S:") + ssid + QLatin1Char(';');
@@ -187,35 +206,27 @@ QString Handler::wifiCode(const QString &connectionPath, const QString &ssid, in
         case NetworkManager::DynamicWep:
         case NetworkManager::WpaEap:
         case NetworkManager::Wpa2Eap:
+        case NetworkManager::Wpa3SuiteB192:
         case NetworkManager::Leap:
-            return {};
+            Q_EMIT wifiCodeReceived(QString(), connectionName);
+            return;
         }
     }
 
     NetworkManager::Connection::Ptr connection = NetworkManager::findConnection(connectionPath);
-    if (!connection)
-        return {};
+    if (!connection) {
+        Q_EMIT wifiCodeReceived(QString(), connectionName);
+        return;
+    }
 
     const auto key = QStringLiteral("802-11-wireless-security");
     auto reply = connection->secrets(key);
-
-    const auto secret = reply.argumentAt<0>()[key];
-    QString pass;
-    switch (securityType) {
-    case NetworkManager::NoneSecurity:
-        break;
-    case NetworkManager::WpaPsk:
-    case NetworkManager::Wpa2Psk:
-    case NetworkManager::SAE:
-        pass = secret[QStringLiteral("psk")].toString();
-        break;
-    default:
-        return {};
-    }
-    if (!pass.isEmpty())
-        ret += QStringLiteral("P:") + pass + QLatin1Char(';');
-
-    return ret + QLatin1Char(';');
+    m_requestWifiCodeWatcher = new QDBusPendingCallWatcher(reply, this);
+    m_requestWifiCodeWatcher->setProperty("key", key);
+    m_requestWifiCodeWatcher->setProperty("ret", ret);
+    m_requestWifiCodeWatcher->setProperty("securityType", static_cast<int>(securityType));
+    m_requestWifiCodeWatcher->setProperty("connectionName", connectionName);
+    connect(m_requestWifiCodeWatcher, &QDBusPendingCallWatcher::finished, this, &Handler::slotRequestWifiCode);
 }
 
 void Handler::addAndActivateConnection(const QString &device, const QString &specificObject, const QString &password)
@@ -268,6 +279,7 @@ void Handler::addAndActivateConnection(const QString &device, const QString &spe
 
     if (securityType == NetworkManager::Leap //
         || securityType == NetworkManager::DynamicWep //
+        || securityType == NetworkManager::Wpa3SuiteB192 //
         || securityType == NetworkManager::Wpa2Eap //
         || securityType == NetworkManager::WpaEap) {
         if (securityType == NetworkManager::DynamicWep || securityType == NetworkManager::Leap) {
@@ -275,6 +287,8 @@ void Handler::addAndActivateConnection(const QString &device, const QString &spe
             if (securityType == NetworkManager::Leap) {
                 wifiSecurity->setAuthAlg(NetworkManager::WirelessSecuritySetting::Leap);
             }
+        } else if (securityType == NetworkManager::Wpa3SuiteB192) {
+            wifiSecurity->setKeyMgmt(NetworkManager::WirelessSecuritySetting::WpaEapSuiteB192);
         } else {
             wifiSecurity->setKeyMgmt(NetworkManager::WirelessSecuritySetting::WpaEap);
         }
@@ -284,9 +298,8 @@ void Handler::addAndActivateConnection(const QString &device, const QString &spe
 
         QPointer<ConnectionEditorDialog> editor = new ConnectionEditorDialog(settings);
         editor->setAttribute(Qt::WA_DeleteOnClose);
+        editor->setWindowFlag(Qt::WindowStaysOnTopHint);
         editor->show();
-        KWindowSystem::setState(editor->winId(), NET::KeepAbove);
-        KWindowSystem::forceActiveWindow(editor->winId());
         connect(editor.data(), &ConnectionEditorDialog::accepted, [editor, device, specificObject, this]() { //
             addAndActivateConnectionDBus(editor->setting(), device, specificObject);
         });
@@ -321,6 +334,57 @@ void Handler::addConnection(const NMVariantMapMap &map)
     connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
 }
 
+struct AddConnectionData {
+    QString id;
+    Handler *handler;
+};
+
+void add_connection_cb(GObject *client, GAsyncResult *result, gpointer user_data)
+{
+    AddConnectionData *data = static_cast<AddConnectionData *>(user_data);
+
+    GError *error = nullptr;
+    NMRemoteConnection *connection = nm_client_add_connection2_finish(NM_CLIENT(client), result, NULL, &error);
+
+    if (error) {
+        KNotification *notification = new KNotification(QStringLiteral("FailedToAddConnection"), KNotification::CloseOnTimeout, data->handler);
+        notification->setTitle(i18n("Failed to add connection %1", data->id));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setText(QString::fromUtf8(error->message));
+        notification->setIconName(QStringLiteral("dialog-warning"));
+        notification->sendEvent();
+
+        g_error_free(error);
+    } else {
+        KNotification *notification = new KNotification(QStringLiteral("ConnectionAdded"), KNotification::CloseOnTimeout, data->handler);
+        notification->setText(i18n("Connection %1 has been added", data->id));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setTitle(data->id);
+        notification->setIconName(QStringLiteral("dialog-information"));
+        notification->sendEvent();
+
+        g_object_unref(connection);
+    }
+
+    delete data;
+}
+
+void Handler::addConnection(NMConnection *connection)
+{
+    NMClient *client = nm_client_new(nullptr, nullptr);
+
+    AddConnectionData *userData = new AddConnectionData{QString::fromUtf8(nm_connection_get_id(connection)), this};
+
+    nm_client_add_connection2(client,
+                              nm_connection_to_dbus(connection, NM_CONNECTION_SERIALIZE_ALL),
+                              NM_SETTINGS_ADD_CONNECTION2_FLAG_IN_MEMORY,
+                              nullptr,
+                              true,
+                              nullptr,
+                              add_connection_cb,
+                              userData);
+}
+
 void Handler::addAndActivateConnectionDBus(const NMVariantMapMap &map, const QString &device, const QString &specificObject)
 {
     QDBusPendingReply<QDBusObjectPath> reply = NetworkManager::addAndActivateConnection(map, device, specificObject);
@@ -885,11 +949,42 @@ void Handler::primaryConnectionTypeChanged(NetworkManager::ConnectionSettings::C
     Q_EMIT hotspotSupportedChanged(m_hotspotSupported);
 }
 
-#if WITH_MODEMMANAGER_SUPPORT
 void Handler::unlockRequiredChanged(MMModemLock modemLock)
 {
     if (modemLock == MM_MODEM_LOCK_NONE) {
         activateConnection(m_tmpConnectionPath, m_tmpDevicePath, m_tmpSpecificPath);
     }
 }
-#endif
+
+void Handler::slotRequestWifiCode(QDBusPendingCallWatcher *watcher)
+{
+    watcher->deleteLater();
+
+    QString ret = watcher->property("ret").toString();
+    const QString connectionName = watcher->property("connectionName").toString();
+    QDBusPendingReply<NMVariantMapMap> reply = *watcher;
+    if (!reply.isValid() || reply.isError()) {
+        Q_EMIT wifiCodeReceived(ret % QLatin1Char(';'), connectionName);
+        return;
+    }
+
+    const auto secret = reply.argumentAt<0>()[watcher->property("key").toString()];
+    QString pass;
+    switch (static_cast<NetworkManager::WirelessSecurityType>(watcher->property("securityType").toInt())) {
+    case NetworkManager::NoneSecurity:
+        break;
+    case NetworkManager::WpaPsk:
+    case NetworkManager::Wpa2Psk:
+    case NetworkManager::SAE:
+        pass = secret[QStringLiteral("psk")].toString();
+        break;
+    default:
+        Q_EMIT wifiCodeReceived(QString(), connectionName);
+        return;
+    }
+    if (!pass.isEmpty()) {
+        ret += QStringLiteral("P:") % pass % QLatin1Char(';');
+    }
+
+    Q_EMIT wifiCodeReceived(ret % QLatin1Char(';'), connectionName);
+}
